@@ -15,7 +15,7 @@ void fill_segment_sse3(const argon2_instance_t *instance, argon2_position_t posi
 
 void fill_segment_ref(const argon2_instance_t *instance, argon2_position_t position);
 
-#ifdef HAVE_SSE3
+#ifdef HAVE_IFUNC
 static void (*resolve_fill_segment(void))(const argon2_instance_t *instance, argon2_position_t position) {
 	__builtin_cpu_init();
 #ifdef HAVE_AVX512
@@ -28,9 +28,11 @@ static void (*resolve_fill_segment(void))(const argon2_instance_t *instance, arg
 		return fill_segment_avx2;
 	else
 #endif
+#ifdef HAVE_SSE3
 	if (__builtin_cpu_supports("sse3"))
 		return fill_segment_sse3;
 	else
+#endif
 	return fill_segment_ref;
 }
 
@@ -38,6 +40,21 @@ void fill_segment(const argon2_instance_t *instance, argon2_position_t position)
      __attribute__ ((ifunc ("resolve_fill_segment")));
 #else
 void fill_segment(const argon2_instance_t *instance, argon2_position_t position) {
+#ifdef HAVE_AVX512
+	if (__builtin_cpu_supports("avx512f"))
+		fill_segment_avx512(instance, position);
+	else
+#endif
+#ifdef HAVE_AVX2
+	if (__builtin_cpu_supports("avx2"))
+		fill_segment_avx2(instance, position);
+	else
+#endif
+#ifdef HAVE_SSE3
+	if (__builtin_cpu_supports("sse3"))
+		fill_segment_sse3(instance, position);
+	else
+#endif
 	fill_segment_ref(instance, position);
 }
 #endif
